library angular2.src.forms.model;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/async.dart'
    show Stream, StreamController, ObservableWrapper;
import 'package:angular2/src/facade/collection.dart'
    show Map, StringMapWrapper, ListWrapper, List;
import './validators.dart' show Validators;

const VALID = "VALID";
const INVALID = "INVALID";
class AbstractControl {
  dynamic _value;
  String _status;
  Map _errors;
  bool _pristine;
  dynamic _parent;
  Function validator;
  Stream valueChanges;
  StreamController _valueChangesController;
  AbstractControl(Function validator) {
    this.validator = validator;
    this._pristine = true;
  }
  dynamic get value {
    return this._value;
  }
  String get status {
    return this._status;
  }
  bool get valid {
    return identical(this._status, VALID);
  }
  Map get errors {
    return this._errors;
  }
  bool get pristine {
    return this._pristine;
  }
  bool get dirty {
    return !this.pristine;
  }
  setParent(parent) {
    this._parent = parent;
  }
  _updateParent() {
    if (isPresent(this._parent)) {
      this._parent._updateValue();
    }
  }
}
class Control extends AbstractControl {
  Control(dynamic value, [Function validator = Validators.nullValidator])
      : super(validator) {
    this._setValueErrorsStatus(value);
    this._valueChangesController = ObservableWrapper.createController();
    this.valueChanges =
        ObservableWrapper.createObservable(this._valueChangesController);
  }
  updateValue(dynamic value) {
    this._setValueErrorsStatus(value);
    this._pristine = false;
    ObservableWrapper.callNext(this._valueChangesController, this._value);
    this._updateParent();
  }
  _setValueErrorsStatus(value) {
    this._value = value;
    this._errors = this.validator(this);
    this._status = isPresent(this._errors) ? INVALID : VALID;
  }
}
class ControlGroup extends AbstractControl {
  Map controls;
  Map _optionals;
  ControlGroup(Map controls,
      [Map optionals = null, Function validator = Validators.group])
      : super(validator) {
    this.controls = controls;
    this._optionals = isPresent(optionals) ? optionals : {};
    this._valueChangesController = ObservableWrapper.createController();
    this.valueChanges =
        ObservableWrapper.createObservable(this._valueChangesController);
    this._setParentForControls();
    this._setValueErrorsStatus();
  }
  include(String controlName) {
    StringMapWrapper.set(this._optionals, controlName, true);
    this._updateValue();
  }
  exclude(String controlName) {
    StringMapWrapper.set(this._optionals, controlName, false);
    this._updateValue();
  }
  bool contains(String controlName) {
    var c = StringMapWrapper.contains(this.controls, controlName);
    return c && this._included(controlName);
  }
  _setParentForControls() {
    StringMapWrapper.forEach(this.controls, (control, name) {
      control.setParent(this);
    });
  }
  _updateValue() {
    this._setValueErrorsStatus();
    this._pristine = false;
    ObservableWrapper.callNext(this._valueChangesController, this._value);
    this._updateParent();
  }
  _setValueErrorsStatus() {
    this._value = this._reduceValue();
    this._errors = this.validator(this);
    this._status = isPresent(this._errors) ? INVALID : VALID;
  }
  _reduceValue() {
    return this._reduceChildren({}, (acc, control, name) {
      acc[name] = control.value;
      return acc;
    });
  }
  _reduceChildren(dynamic initValue, Function fn) {
    var res = initValue;
    StringMapWrapper.forEach(this.controls, (control, name) {
      if (this._included(name)) {
        res = fn(res, control, name);
      }
    });
    return res;
  }
  bool _included(String controlName) {
    var isOptional = StringMapWrapper.contains(this._optionals, controlName);
    return !isOptional || StringMapWrapper.get(this._optionals, controlName);
  }
}
class ControlArray extends AbstractControl {
  List controls;
  ControlArray(List<AbstractControl> controls,
      [Function validator = Validators.array])
      : super(validator) {
    this.controls = controls;
    this._valueChangesController = ObservableWrapper.createController();
    this.valueChanges =
        ObservableWrapper.createObservable(this._valueChangesController);
    this._setParentForControls();
    this._setValueErrorsStatus();
  }
  AbstractControl at(num index) {
    return this.controls[index];
  }
  push(AbstractControl control) {
    ListWrapper.push(this.controls, control);
    control.setParent(this);
    this._updateValue();
  }
  insert(num index, AbstractControl control) {
    ListWrapper.insert(this.controls, index, control);
    control.setParent(this);
    this._updateValue();
  }
  removeAt(num index) {
    ListWrapper.removeAt(this.controls, index);
    this._updateValue();
  }
  num get length {
    return this.controls.length;
  }
  _updateValue() {
    this._setValueErrorsStatus();
    this._pristine = false;
    ObservableWrapper.callNext(this._valueChangesController, this._value);
    this._updateParent();
  }
  _setParentForControls() {
    ListWrapper.forEach(this.controls, (control) {
      control.setParent(this);
    });
  }
  _setValueErrorsStatus() {
    this._value = ListWrapper.map(this.controls, (c) {
      return c.value;
    });
    this._errors = this.validator(this);
    this._status = isPresent(this._errors) ? INVALID : VALID;
  }
}
